package org.topbraid.spin.model;

import java.util.List;

import org.topbraid.spin.model.print.Printable;

import com.hp.hpl.jena.rdf.model.Resource;


/**
 * Base interface of the various SPARQL query types such as
 * Ask, Construct, Describe and Select.
 * 
 * @author Holger Knublauch
 */
public interface Query extends Printable, Resource {
	
	
	/**
	 * Gets the comment if any has been stored as rdfs:comment.
	 * @return the comment or null
	 */
	String getComment();
	
	
	/**
	 * Gets the list of URIs specified in FROM clauses.
	 * @return a List of URI Strings
	 */
	List<String> getFrom();
	
	
	/**
	 * Gets the list of URIs specified in FROM NAMED clauses.
	 * @return a List of URI Strings
	 */
	List<String> getFromNamed();
	
	
	/**
	 * Gets the ElementList of the WHERE clause of this query.
	 * Might be null or RDF.nil.
	 * @return the WHERE clause as an ElementList
	 */
	ElementList getWhere();

	
	/**
	 * Gets the elements in the WHERE clause of this query.
	 * The Elements will be typecast into the best suitable subclass.
	 * @return a List of Elements
	 */
	List<Element> getWhereElements();
}
