package org.topbraid.spin.model.impl;

import java.util.LinkedList;
import java.util.List;

import org.topbraid.spin.model.Element;
import org.topbraid.spin.model.ElementList;
import org.topbraid.spin.model.SPINFactory;
import org.topbraid.spin.model.TriplePattern;
import org.topbraid.spin.model.Variable;
import org.topbraid.spin.model.print.PrintContext;
import org.topbraid.spin.model.print.StringPrintContext;
import org.topbraid.spin.model.visitor.ElementVisitor;

import com.hp.hpl.jena.enhanced.EnhGraph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.impl.RDFListImpl;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.vocabulary.RDF;


public class ElementListImpl extends RDFListImpl implements ElementList {

	public ElementListImpl(Node node, EnhGraph graph) {
		super(node, graph);
	}

	
	public List<Element> getElements() {
		List<Element> results = new LinkedList<Element>();
		ExtendedIterator it = iterator();
		while(it.hasNext()) {
			RDFNode node = (RDFNode) it.next();
			if(node.isResource()) {
				Element element = SPINFactory.asElement((Resource)node);
				if(element != null) {
					results.add(element);
				}
			}
		}
		return results;
	}
	
	
	/*
	private Map<Variable,List<RDFNode>> getNestedLists(List<Element> elements, Set<TriplePattern> suppressedPatterns) {
		Map<Variable,List<RDFNode>> results = new HashMap<Variable,List<RDFNode>>();
		for(int i = 0; i < elements.size() - 1; i++) {
			List<RDFNode> members = null;
			while(i < elements.size() - 1 && 
					elements.get(i) instanceof TriplePattern && 
					elements.get(i + 1) instanceof TriplePattern) {
				TriplePattern firstPattern = (TriplePattern) elements.get(i);
				TriplePattern secondPattern = (TriplePattern) elements.get(i + 1);
				if(RDF.first.equals(firstPattern.getPredicate()) && RDF.rest.equals(secondPattern.getPredicate())) {
					Resource subject = firstPattern.getSubject();
					Resource secondSubject = secondPattern.getSubject();
					if(subject instanceof Variable && secondSubject instanceof Variable) {
						Variable bnodeVar = (Variable) subject;
						Variable secondVar = (Variable) secondPattern;
						if(bnodeVar.isBlankNodeVar() && bnodeVar.getName().equals(secondVar.getName())) {
							if(members == null) {
								members = new LinkedList<RDFNode>();
								results.put(bnodeVar, members);
							}
							members.add(firstPattern.getObject());
							RDFNode secondObject = secondPattern.getObject();
							i++;
							if(RDF.nil.equals(secondObject)) {
								members = null;
								break;
							}
						}
					}
				}
				i++;
			}
		}
		return results;
	}*/
	
	
	public void print(PrintContext p) {
		List<Element> elements = getElements();
		
		//Set<TriplePattern> suppressedPatterns = new HashSet<TriplePattern>();
		//Map<Variable,List<RDFNode>> lists = getNestedLists(elements, suppressedPatterns);
		
		int oldI = -1;
		for(int i = 0; i < elements.size(); i++) {
			if(i == oldI) {
				break; // Prevent unknown endless loop conditions
			}
			oldI = i;
			Element element = elements.get(i);
			p.printIndentation(p.getIndentation());
			if(element instanceof ElementList) {
				p.print("{");
				p.println();
				p.setIndentation(p.getIndentation() + 1);
				element.print(p);
				p.setIndentation(p.getIndentation() - 1);
				p.printIndentation(p.getIndentation());
				p.print("}");
			}
			else {
				
				TriplePattern tp = null;
				int x = i;
				
				if(element instanceof TriplePattern) {
					TriplePattern t = (TriplePattern) element;
					if(t.getObject() instanceof Variable && !RDF.first.equals(t.getPredicate())) {
						x++;
						tp = (TriplePattern) t;
					}
				}
				
				// Special handling of rdf:Lists in TriplePatterns
				List<RDFNode> members = new LinkedList<RDFNode>();
				Variable listVar = null;
				while(x < elements.size() - 1 &&
						elements.get(x) instanceof TriplePattern && 
						elements.get(x + 1) instanceof TriplePattern) {
					boolean found = false;
					TriplePattern firstPattern = (TriplePattern) elements.get(x);
					if(RDF.first.equals(firstPattern.getPredicate())) {
						TriplePattern secondPattern = (TriplePattern) elements.get(x + 1);
						if(RDF.rest.equals(secondPattern.getPredicate())) {
							Resource subject = firstPattern.getSubject();
							if(subject instanceof Variable) {
								Variable bnodeVar = (Variable) subject;
								if(bnodeVar.isBlankNodeVar()) {
									if(listVar == null) {
										listVar = bnodeVar;
									}
									if(bnodeVar.equals(secondPattern.getSubject())) {
										members.add(firstPattern.getObject());
										x += 2;
										found = true;
									}
								}
							}
						}
					}
					if(!found) {
						break;
					}
				}
				
				if(members.isEmpty()) {
					element.print(p);
				}
				else {
					boolean listPrinted = false;
					if(tp != null) { // We have <a> <b> (...)
						if(tp.getObject().equals(listVar)) {
							TupleImpl.print(getModel(), tp.getSubject(), p);
							p.print(" ");
							TupleImpl.print(getModel(), tp.getPredicate(), p);
							p.print(" ");
							printRDFList(p, members);
							listPrinted = true;
							i = x - 1;
						}
						else {
							i--;
						}
					}
					else if(x < elements.size()) {
						if(elements.get(x) instanceof TriplePattern) {
							tp = (TriplePattern)elements.get(x);
							if(tp.getSubject().equals(listVar)) { // We have (...) <b> <c>
								RDFNode object = tp.getObject();
								// Avoid cases where both are lists -> Too complicating!
								if(!(object instanceof Variable) || !((Variable)object).isBlankNodeVar()) {
									printRDFList(p, members);
									p.print(" ");
									TupleImpl.print(getModel(), tp.getPredicate(), p);
									p.print(" ");
									TupleImpl.print(getModel(), tp.getObject(), p);
									listPrinted = true;
									i = x;
								}
								else {
									element.print(p);
									listPrinted = true;
								}
							}
						}
						else {
							i--;
						}
					}
					if(!listPrinted) {
						printRDFList(p, members);
					}
				}
			}
			p.print(" .");
			p.println();
		}
	}
	
	
	private void printRDFList(PrintContext p, List<RDFNode> members) {
		p.print("(");
		for(RDFNode node : members) {
			p.print(" ");
			TupleImpl.print(getModel(), node, p);
		}
		p.print(" )");
	}
	
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		PrintContext context = new StringPrintContext(sb);
		print(context);
		return sb.toString();
	}


	public void visit(ElementVisitor visitor) {
		visitor.visit(this);
	}
}