package org.topbraid.spin.model.impl;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.topbraid.spin.model.Select;
import org.topbraid.spin.model.Variable;
import org.topbraid.spin.model.print.PrintContext;
import org.topbraid.spin.vocabulary.SP;

import com.hp.hpl.jena.enhanced.EnhGraph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.rdf.model.RDFList;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.vocabulary.RDF;


public class SelectImpl extends QueryImpl implements Select {

	public SelectImpl(Node node, EnhGraph eh) {
		super(node, eh);
	}

	
	public List<Variable> getResultVariables() {
		List<Variable> results = new LinkedList<Variable>();
		for(RDFNode node : getList(SP.resultVariables)) {
			results.add((Variable)node.as(Variable.class));
		}
		return results;
	}
	
	
	public boolean isDistinct() {
		return hasProperty(SP.distinct, getModel().createTypedLiteral(true));
	}
	
	
	public boolean isReduced() {
		return hasProperty(SP.reduced, getModel().createTypedLiteral(true));
	}


	public void print(PrintContext context) {
		printComment(context);
		printPrefixes(context);
		context.printKeyword("SELECT");
		context.print(" ");
		if(isDistinct()) {
			context.printKeyword("DISTINCT");
			context.print(" ");
		}
		if(isReduced()) {
			context.printKeyword("REDUCED");
			context.print(" ");
		}
		List<Variable> vars = getResultVariables();
		if(vars.isEmpty()) {
			context.print("*");
		}
		else {
			for(Iterator<Variable> vit = vars.iterator(); vit.hasNext(); ) {
				Variable var = vit.next();
				if(var.hasProperty(RDF.type, SP.Count) || var.hasProperty(RDF.type, SP.Sum)) {
					boolean as = false;
					String varName = "*";
					Statement varS = var.getProperty(SP.expression);
					if(varS != null && varS.getObject().isResource()) {
						Variable nestedVar = (Variable) varS.getResource().as(Variable.class);
						varName = nestedVar.toString();
					}
					
					Statement asS = var.getProperty(SP.as);
					if(asS != null && asS.getObject().isResource()) {
						as = true;
						context.print("(");
					}
					if(var.hasProperty(RDF.type, SP.Count)) {
						context.printKeyword("COUNT");
					}
					else {
						context.printKeyword("SUM");
					}
					context.print("(");
					context.print(varName);
					if(as) {
						context.print(") ");
						context.printKeyword("AS");
						context.print(" ");
						Variable asVar = (Variable) asS.getResource().as(Variable.class);
						context.print(asVar.toString());
					}
					context.print(")");
				}
				else {
					var.print(context);
					// context.print(var.toString());
				}
				if(vit.hasNext()) {
					context.print(" ");
				}
			}
		}
		printStringFrom(context);
		context.println();
		printWhere(context);
		printGroupBy(context);
		printSolutionModifiers(context);
	}
	
	
	private void printGroupBy(PrintContext p) {
		Statement groupByS = getProperty(SP.groupBy);
		if(groupByS != null) {
			RDFList list = (RDFList) groupByS.getObject().as(RDFList.class);
			ExtendedIterator it = list.iterator();
			if(it.hasNext()) {
				p.printIndentation(p.getIndentation());
				p.printKeyword("GROUP BY");
				while(it.hasNext()) {
					p.print(" ");
					RDFNode node = (RDFNode) it.next();
					printNestedExpressionString(p, node);
				}
			}
		}
	}
}
