package org.topbraid.spin.system;

import com.hp.hpl.jena.rdf.model.Resource;


/**
 * A singleton that is used to render resources into strings.
 * By default this displays qnames (if possible). 
 * Can be changed, for example, to switch to displaying rdfs:labels
 * instead of qnames etc.
 * 
 * @author Holger Knublauch
 */
public class SPINLabels {
	
	private static SPINLabels singleton = new SPINLabels();
	

	/**
	 * Gets the singleton instance of this class.
	 * @return the singleton
	 */
	public static SPINLabels get() {
		return singleton;
	}
	
	
	/**
	 * Replaces the singleton to a subclass with different behavior.
	 * This is used by TopBraid, which has its own rendering engine. 
	 * @param value  the new engine
	 */
	public static void set(SPINLabels value) {
		SPINLabels.singleton = value;
	}
	

	/**
	 * Gets the label for a given Resource.
	 * @param resource  the Resource to get the label of
	 * @return the label (never null)
	 */
	public String getLabel(Resource resource) {
		if(resource.isURIResource()) {
			String qname = resource.getModel().qnameFor(resource.getURI());
			if(qname != null) {
				return qname;
			}
			else {
				return "<" + resource.getURI() + ">";
			}
		}
		else {
			return resource.toString();
		}
	}
}
