package org.topbraid.spin.util;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.topbraid.spin.internal.JenaUtil;
import org.topbraid.spin.model.Ask;
import org.topbraid.spin.model.Construct;
import org.topbraid.spin.model.SPINFactory;
import org.topbraid.spin.model.Template;
import org.topbraid.spin.model.TemplateCall;
import org.topbraid.spin.model.print.StringPrintContext;
import org.topbraid.spin.system.ARQFactory;
import org.topbraid.spin.system.SPINLabels;
import org.topbraid.spin.vocabulary.SPIN;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;


public class SPINQueryFinder {
	

	public static void add(Map<Resource, List<QueryWrapper>> class2Query, Statement s,
			Model model,
			boolean withClass,
			Map<QueryWrapper,
			Map<String, RDFNode>> initialTemplateBindings,
			boolean allowAsk) {
		if(s.getObject().isResource()) {
			String ruleText = null;
			String label = null;
			org.topbraid.spin.model.Query rule = null;
			Template template = null;
			TemplateCall templateCall = SPINFactory.asTemplateCall(s.getResource());
			if(templateCall != null) {
				template = templateCall.getTemplate();
				if(template != null) {
					org.topbraid.spin.model.Query body = template.getBody();
					if(body instanceof Construct || (allowAsk && body instanceof Ask)) {
						rule = body;
					}
				}
				ruleText = SPINLabels.get().getLabel(templateCall);
				label = ruleText;
			}
			else {
				rule = SPINFactory.asQuery(s.getResource());
				if(rule != null) {
					label = rule.getComment();
				}
			}
			
			if(rule != null) {
				StringPrintContext p = new StringPrintContext();
				rule.print(p);
				if(ruleText == null) {
					ruleText = p.getString();
				}
				if(withClass && rule instanceof Construct && SPINUtil.containsThis(rule)) {
					SPINUtil.addThisTypeClause(p.getStringBuilder());
				}
				QueryWrapper wrapper = null;
				Query arqQuery = ARQFactory.get().createQuery(model, p.getStringBuilder().toString());
				if(arqQuery.isConstructType() || (allowAsk && arqQuery.isAskType())) {
					Resource type = s.getSubject();
					List<QueryWrapper> list = class2Query.get(type);
					if(list == null) {
						list = new LinkedList<QueryWrapper>(); 
						class2Query.put(type, list);
					}
					wrapper = new QueryWrapper(arqQuery, ruleText, rule, label);
					list.add(wrapper);
				}
				
				if(template != null && wrapper != null) {
					Map<String,RDFNode> bindings = templateCall.getArgumentsMapByVarNames();
					if(!bindings.isEmpty()) {
						initialTemplateBindings.put(wrapper, bindings);
					}
				}
			}
		}
	}

	
	/**
	 * Gets a Map of QueryWrappers with their associated classes. 
	 * @param model  the Model to operate on
	 * @param queryModel  the Model to query on (might be different)
	 * @param predicate  the predicate such as <code>spin:rule</code>
	 * @param withClass  true to also include a SPARQL clause to bind ?this
	 *                   (something along the lines of ?this a ?THIS_CLASS) 
	 * @param initialTemplateBindings  will contain the initial bindings if
	 *                                 QueryWrappers wrap SPIN template calls
	 * @param allowAsk  also return ASK queries
	 * @return the result Map, possibly empty but not null
	 */
	public static Map<Resource, List<QueryWrapper>> getClass2QueryMap(Model model, Model queryModel, Property predicate, boolean withClass, Map<QueryWrapper,Map<String,RDFNode>> initialTemplateBindings, boolean allowAsk) {
		predicate = model.getProperty(predicate.getURI());
		Map<Resource,List<QueryWrapper>> class2Query = new HashMap<Resource,List<QueryWrapper>>();
		List<Statement> ss = JenaUtil.getStatementsList(JenaUtil.listAllProperties(null, predicate));
		Collections.sort(ss, new Comparator<Statement>() {
			public int compare(Statement o1, Statement o2) {
				if(o1.getObject().isResource() && o2.getObject().isResource()) {
					Integer index1 = SPIN.getRuleIndex(o1.getResource());
					Integer index2 = SPIN.getRuleIndex(o2.getResource());
					return index1.compareTo(index2);
				}
				else {
					return 0;
				}
			}
		});
		for(Statement s : ss) {
			add(class2Query, s, model, withClass, initialTemplateBindings, allowAsk);
		}
		return class2Query;
	}

}
