package org.topbraid.spin.vocabulary;

import java.io.InputStream;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;


/**
 * Vocabulary of the SPIN Modeling Vocabulary.
 * 
 * @author Holger Knublauch
 */
public class SPIN {

	public final static String BASE_URI = "http://spinrdf.org/spin";
	
	public final static String NS = BASE_URI + "#";
	
	public final static String PREFIX = "spin";
	
	
	public final static String THIS_VAR_NAME = "this";


	public final static Resource Function = ResourceFactory.createResource(NS + "Function");

	public final static Resource Functions = ResourceFactory.createResource(NS + "Functions");

	public final static Resource LibraryOntology = ResourceFactory.createResource(NS + "LibraryOntology");

	public final static Resource Module = ResourceFactory.createResource(NS + "Module");

	public final static Resource Modules = ResourceFactory.createResource(NS + "Modules");

	public final static Resource Template = ResourceFactory.createProperty(NS + "Template");

	public final static Resource Templates = ResourceFactory.createProperty(NS + "Templates");

	public final static Resource ConstraintViolation = ResourceFactory.createProperty(NS + "ConstraintViolation");

	
	public final static Property abstract_ = ResourceFactory.createProperty(NS + "abstract");
	
	public final static Property body = ResourceFactory.createProperty(NS + "body");
	
	public final static Property constraint = ResourceFactory.createProperty(NS + "constraint");
	
	public final static Property constructor = ResourceFactory.createProperty(NS + "constructor");

	public final static Property labelTemplate = ResourceFactory.createProperty(NS + "labelTemplate");

	public final static Property query = ResourceFactory.createProperty(NS + "query");

	public final static Property returnType = ResourceFactory.createProperty(NS + "returnType");
	
	public final static Property rule = ResourceFactory.createProperty(NS + "rule");

	public final static Property ruleIndex = ResourceFactory.createProperty(NS + "ruleIndex");

	public final static Property symbol = ResourceFactory.createProperty(NS + "symbol");
	
	public final static Property violationPath = ResourceFactory.createProperty(NS + "violationPath");
	
	public final static Property violationRoot = ResourceFactory.createProperty(NS + "violationRoot");
	

	public final static Resource _arg1 = ResourceFactory.createProperty(NS + "_arg1");

	public final static Resource _arg2 = ResourceFactory.createProperty(NS + "_arg2");

	public final static Resource _arg3 = ResourceFactory.createProperty(NS + "_arg3");

	public final static Resource _arg4 = ResourceFactory.createProperty(NS + "_arg4");

	public final static Resource _arg5 = ResourceFactory.createProperty(NS + "_arg5");
	
	public final static Resource _this = ResourceFactory.createResource(NS + "_this");

	
	static {
		// Force initialization
		SP.getURI();
	}
	
	
	private static Model model;
	

	/**
	 * Gets a Model with the content of the SPIN namespace, from a file
	 * that is bundled with this API.
	 * @return the namespace Model
	 */
	public static Model getModel() {
		if(model == null) {
			model = ModelFactory.createDefaultModel();
			InputStream is = SPIN.class.getClassLoader().getResourceAsStream("/etc/spin.owl");
			model.read(is, "http://dummy");
		}
		return model;
	}

	
	public static int getRuleIndex(Resource rule) {
		Statement s = rule.getProperty(SPIN.ruleIndex);
		if(s != null && s.getObject().isLiteral()) {
			return s.getInt();
		}
		else {
			return 100000;
		}
	}
}
